// Estimation of a mixture for classification (in T73)
// - data from Skoda car used
// - initialization with MANUALLY SET INITIAL COMPONENT CENTERS
[u,t,n]=file();                         // find working directory
chdir(dirname(n(2)));                   // set working directory
clear("u","t","n")                      // clear auxiliary data
exec("ScIntro.sce",-1),mode(0)          // intro to sesion                 

nd=600;
T=100;                   // number of measurements in a sampling T 
I_init_showClus=0;       // =0: manulally set initial clusters
                         // =1: show data clusters for analysis
// =========================== DATA
load _data/dataKF.dat    // data load
dt=DataKF.dataOrig;

cc=dt(6,:); // gear and its discretization
c5=zeros(1,length(cc));
j=find((cc==-1)|(cc==0)); c5(j)=1;
j=find((cc>.5)&(cc<3.5)); c5(j)=2;
j=find(cc==4); c5(j)=3;
j=find(cc==5); c5(j)=4;
j=find(cc==6); c5(j)=5;

ct=c5(1:T:$);
Sim.nc=max(ct);
yt=[dt(2,1:T:$);              // speed
dt(4,1:T:$);                  // gas
dt(7,1:T:$);                  // moment
dt(8,1:T:$)/1000];            // revs
Sim.Cy.my=size(yt,1);
Sim.Cy.yt=yt;
Sim.Cy.T=T;                   // model period of sampling (in T0)
Sim.Cy.T0='5 ms';             // basic period of sampling
Sim.Cp.yt=ct;
nc=Sim.nc;
my=Sim.Cy.my;

// =========================== INICIALIZACE
printf(' Initialization ...\n')
if I_init_showClus==1
  set(scf(2),'position',[700 50 800 600])
  for i=1:my
    for j=(i):my
      subplot(my,my,my*(i-1)+j)
      if i==j
        [f,x]=nan_hist(yt(i,:),40);
        bar(x,f)
        x1=min(x); x2=max(x);
        ll=(x1:(x2-x1)/10:x2)';
        loc=round(ll);
        lab=string(loc);
        a=gca();
        a.x_ticks = tlist(["ticks", "locations", "labels"], loc, lab);
      else            
        plot(yt(i,:),yt(j,:),'.','markersize',3)
      end
    end
  end
  return
end

// Manually set - determined from data plots
printf(' ***************************************************\n')
printf(' The initial clusters are assumed to be manually set\n')
printf(' ***************************************************\n')
Est.Cy(1).th=[0 0 -20 1]';
Est.Cy(2).th=[50 31 0 1.3]';
Est.Cy(3).th=[80 45 40 1.7]';
Est.Cy(4).th=[125 60 80 2.4]';
Est.Cy(5).th=[120 30 40 2.1]'; // pridano

// initial point estimates
a=.8;  // scattering around the set points
for j=1:nc
  [mr,mc]=size(Est.Cy(j).th);
  Ps=[Est.Cy(j).th;1]+[a*rand(mr,mc,'n');0];
  Est.Cy(j).V=Ps*Ps';
  Est.Cy(j).cv=.1*eye(my,my);
end

Est.ka=ones(1,nc);                 // data counter
Est.Cp.V=ones(1,nc);               // pointer statistics
Est.Cp.th=fnorm(rand(1,nc,'u')+1); // pointer parameter estimate
w=fnorm(ones(1,nc));               // weights

// =========================== TIME LOOP ==============================
printf( ' Time loop ...\n')
ce=ones(1,nd);
printf(' '), tt=fix(nd/10); 
for t=(2:nd)
  if t/tt==fix(t/tt), printf('.'); end
  for j=1:nc
    [xxx,G(j)]=GaussN(yt(:,t),Est.Cy(j).th,Est.Cy(j).cv); //datova predikce
  end
  Lq=G-max(G);
  q=exp(Lq);

  ww=q'.*Est.Cp.th; w=ww/sum(ww); // weight construction
  wt(:,t)=w';

  Ps=[yt(:,t)' 1]';                          // ext. reg. vector
  //update statistic 
  for i=1:nc
    Est.Cy(i).V=Est.Cy(i).V+w(i)*Ps*Ps';     // update - data stat.
    Est.ka(i)=Est.ka(i)+w(i);                // update - counter
    Est.Cp.V(i)=Est.Cp.V(i)+w(i);            //update - pt.stat.     

    //nove rozdeleni informacni matice V
    Vyy=Est.Cy(i).V(1:my,1:my);              // part Vyy 
    Vy=Est.Cy(i).V($,1:my);                  // part Vy
    V1=Est.Cy(i).V($,$);                     // part V1
    Est.Cy(i).th=inv(V1+1e-8*eye(V1))*Vy;    // pt.est. of reg. coef.
    Est.Cy(i).tht(:,t)=Est.Cy(i).th';        // store
    if 0 // 0=all with initial covariances ( recommended )
      Est.Cy(i).cv=(Vyy-Vy'*inv(V1+1e-8*eye(V1))*Vy)/Est.ka(i);
    end
  end
  Est.Cp.th=fnorm(Est.Cp.V,2);               // pt.est. - pointer par. 

  [xxx,ce(t)]=max(w);                        // pt.est. of the pointer

  // mixture prediction
  yy=zeros(my,1);
  for i=1:nc
    yy=yy+w(i)*Est.Cy(i).th';
  end
  yp(:,t)=yy;
end // ======================= END OF TIME LOOP =======================
Est.Cp.yt=ce;
Est.yp=yp;

// RESULTS
s=(nd-20):nd;
set(scf(1),'position',[100 100 600 500])
for i=1:nc
  subplot(nc,1,i)
  plot(Est.Cy(i).tht')
end

s=1:length(ce);
//q=1:nc; // 
[q,T]=c2c(ct(s),ce');
set(scf(2),'position',[500 100 600 500])
plot(q(ce(s)),':x')
plot(ct(s),'r:.','markersize',5)
title 'Class point estimates'

s=(fix(nd/2)+1):nd;
set(scf(3),'position',[750 200 600 500])
for i=1:my
  subplot(nc,1,i)
  plot(s,yt(i,s),'b',s,yp(i,s),'r')
  legend('output','prediction');
end

save _data/dataT72.dat Est Sim
